﻿using NAisParser;
using System;
using System.Diagnostics;
using System.IO;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace Madart.OperationalUI.AIS
{
	public class AISListener
	{
		public bool stopStream;

		private int recievedMessages;
		private readonly DateTime start;

		public event EventHandler<AisMessageEventArgs> OnMessage;
		public event EventHandler<AisUpdateEventArgs> OnUpdate;

		public AISListener()
		{
			stopStream = false;
			recievedMessages = 0;
			start = DateTime.Now;
		}

		/// <summary>
		/// Starts and continues to reconnect to an online AIS-stream and triggers events that VesselUpdater is listening for.
		/// </summary>
		/// <param name="server"> IP of AIS server</param>
		/// <param name="port"> Relevant port to the AIS server</param>
		/// <returns></returns>
		public Task StartStreamListener(string server, int port)
		{
			var reader = GetAisStreamReader(server, port);
			using (var parser = new AISParser())
			{
				try
				{
					while (!stopStream)
					{
						var messageStr = reader.ReadLine();
						if (messageStr != null && messageStr != "")
						{

							

							if (!string.IsNullOrWhiteSpace(messageStr))
							{
								var message = parser.ParseMessage(messageStr);
								if (message != null)
								{

									if (message is MessageType123)
									{
										var messageNo = recievedMessages++;

										OnMessage.Invoke(this, new AisMessageEventArgs { MessageNo = messageNo, Message = (NAisParser.MessageType123)message, TimeStamp = DateTime.Now });

									}else if(message is MessageType5)
									{
										OnUpdate.Invoke(this, new AisUpdateEventArgs {Message = (MessageType5)message, TimeStamp = DateTime.Now });
									}


								}
							}
						}
						else if (messageStr == null)
						{
							// Create a new reader
							Debug.WriteLine("NullMessage, reconnecting to stream");
							Thread.Sleep(500);
							reader = GetAisStreamReader(server, port);
						}
					}
				}
				catch (Exception e)
				{
					Debug.WriteLine(e.StackTrace);
				}
				finally
				{
					Thread.Sleep(5000); // Sleep 5 seconds in case this fails immediately to prevent locking up the CPU and filling the log
					StartStreamListener(server, port);
				}
			}

			return Task.CompletedTask;
		}

		private static StreamReader GetAisStreamReader(string server, int port)
		{
			var client = new TcpClient(server, port);
			var reader = new StreamReader(client.GetStream(), Encoding.UTF8);
			return reader;
		}

		public double MessagesDeltaTime()
		{
			var now = DateTime.UtcNow;
			if (now.CompareTo(start) > 0)
			{
				var elapsedTicks = now.Ticks - start.Ticks;
				var elapsedSpan = new TimeSpan(elapsedTicks);
				return recievedMessages / elapsedSpan.TotalSeconds;
			}
			return 0.0;
		}
	}

	public class AisMessageEventArgs : EventArgs
	{
		public int MessageNo { get; internal set; }
		public MessageType123 Message { get; internal set; }
		public DateTime TimeStamp { get; internal set; }
	}
	public class AisUpdateEventArgs : EventArgs
	{
		public MessageType5 Message { get; internal set; }
		public DateTime TimeStamp { get; internal set; }
	}
}

