﻿using Madart.OperationalUI.DataAccess.Models;
using Madart.OperationalUI.DataAccess.Repositories;
using Madart.OperationalUI.DataAccess.Repositories.Interface;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Madart.OperationalUI.DataAccess.Repositories
{
	public interface IVesselStateRepository
	{
		List<VesselState> GetHistory(int mmsi, int length = 400);
		List<VesselState> GetAllVessels();
		void CleanVesselStates(DateTime boundaryTime);
	}
	public class VesselStateRepository : AbstractRepository<VesselState>, IVesselStateRepository
	{


		/// <summary>
		/// Retrieve the historic list of VesselStates for the vessel identified by the given MMSI
		/// </summary>
		/// <param name="mmsi">The MMSI number that identifies the vessel</param>
		/// <param name="length">The maximum number of states to return. Default is 400.</param>
		/// <returns>List of VesselStates as a history</returns>
		public List<VesselState> GetHistory(int mmsi, int length = 400)
		{
			var list = (from vessel in Context.VesselState
							where vessel.MmsiNo == mmsi
								orderby vessel.VesselStateId descending
									select vessel).Take(length).ToList();
			return list;
		}


		/// <summary>
		/// Retrieves a List of all VesselStates from the DB
		/// </summary>
		/// <returns>List of all VesselStates in the DB</returns>
		public List<VesselState> GetAllVessels()
		{
			return Context.VesselState.GroupBy(v => v.MmsiNo).Select(g => g.OrderByDescending(g2 => g2.VesselStateId).First()).ToList();

		}

		/// <summary>
		/// Cleans the DB from VesselStates older then a boundaryTime
		/// </summary>
		/// <param name="boundaryTime">Every VesselState in the DB before this time will be removed</param>
		public void CleanVesselStates(DateTime boundaryTime)
		{
			Context.VesselState.RemoveRange(Context.VesselState.Where(vessel => vessel.TimeReceived < boundaryTime));
			Context.SaveChanges();
		}
	}
}
