﻿using System;
using System.Net;
using System.Text;
using System.IO;
using System.Diagnostics;
using System.Collections.Generic;
using Newtonsoft.Json;
using System.Linq;
using System.Threading.Tasks;
using System.Data;
using Madart.OperationalUI.DataAccess.Models;
using Madart.OperationalUI.MadAPI.Models;
using System.Threading;
using System.Net.Http;

namespace Madart.OperationalUI.MadAPI
{

	public static class MadApiClient
	{
		static int flip = 0;
		private static string GetBaseUrl()
		{
			var urls = new string[] { "http://172.19.10.185:5000" }; // , "http://172.19.3.25:5000 http://172.19.10.185:5000  http://172.19.3.25:5000"
            var idx = (flip++ % urls.Length);
			return urls[idx];
		}

		/// <summary>
		/// General Post request, can be used for all different post urls
		/// </summary>
		/// <param name="url">The request Url ex. http://172.19.10.185:5000/api/get_routes </param>
		/// <param name="value">The Json object you want to send to the api.</param>
		/// <returns>Returns the response from the API in Json format</returns>

		public static async Task<string> Post(Uri url, string value)
		{
			ServicePointManager.DefaultConnectionLimit = 1000;
			ServicePointManager.ReusePort = false;
			WebRequest.DefaultWebProxy = null;

			var request = HttpWebRequest.CreateHttp(url);
			string result = "";
			request.Proxy = null;
			request.ConnectionGroupName = "MadAPI";
			request.Timeout = 120000;
			request.KeepAlive = false;
			request.Proxy = null;
			request.ServicePoint.ConnectionLeaseTimeout = 50;
			request.ServicePoint.MaxIdleTime = 50;

			var byteData = Encoding.ASCII.GetBytes(value);
			request.ContentType = "application/json";
			request.Method = "POST";

			try
			{
				using (var stream = request.GetRequestStream())
				{
					stream.Write(byteData, 0, byteData.Length);
				}
				using (var response = (HttpWebResponse)request.GetResponse())
				using (var responseStream = response.GetResponseStream())
				using (var reader = new StreamReader(responseStream))
				{
					result = reader.ReadToEnd();
					return result;
				}
			}
			catch (Exception e)
			{
				Debug.WriteLine("MadAPI request failed because: " + e.Message);
				Debug.WriteLine("Payload:");
				Debug.WriteLine(value);
				Debug.WriteLine($"Sent to {url.ToString()}");
				Debug.WriteLine(result);
				return null;
			}
		}

		/// <summary>
		/// Gets all normal routes classified to a vessel given its history
		/// </summary>
		/// <param name="vesselstates">List of AIS-messages for the boat you want the normalroutes for</param>
		/// <returns>List of NormalRoutes, returns null if no route is found</returns>
		public static async Task<List<Tuple<int?, double?>>> GetNormalRoutesAsync(List<VesselState> vesselstates, bool prioritizeDtwDistance = false)
		{
			TrajectoryHistory item = new TrajectoryHistory();

			item.prioritize_dtw_distance = prioritizeDtwDistance;

			float? latestCos = (float)vesselstates[vesselstates.Count - 1].CourseOverGround;
			List<float?> listOfSog = new List<float?>();
			List<List<float>> listOfPos = new List<List<float>>();
			for (int i = 0; i < vesselstates.Count(); i++)
			{
				List<float> coord = new List<float>();
				coord.Add((float)vesselstates[i].Position.X);
				coord.Add((float)vesselstates[i].Position.Y);
				listOfPos.Add(coord);
			}


			for (int i = 0; i < vesselstates.Count; i++)
			{
				listOfSog.Add((float)(vesselstates[i].SpeedOverGround/10.0));

			}

			item.trajectory = listOfPos;
			item.sog = listOfSog;
			item.cog = latestCos;


			string request = JsonConvert.SerializeObject(item);
			Uri url = new Uri(string.Format(GetBaseUrl() + "/api/get_routes"));

			string response = await Post(url, request);
			if (response == null)
			{
				Debug.WriteLine("Unable to send request to MadApi @ " + url.ToString());
				return null;
			}

			if (response.Contains("result"))
			{
				return null;
			}
			else
			{
				try
				{
					List<MadAPI.Models.NormalRoute> list = JsonConvert.DeserializeObject<List<MadAPI.Models.NormalRoute>>(response);
					List<Tuple<int?, double?>> rankList = new List<Tuple<int?, double?>>();
					int counter = list.Count > 5 ? 5 : list.Count;
					for (int i = 0; i < counter; i++)
					{
						rankList.Add(new Tuple<int?, double?>(list[i].normal_route_id, list[i].ranking_score));
					}
					return rankList;
				}
				catch (JsonSerializationException)
				{

				}
			}
			return null;
		}

		/// <summary>
		/// Fetches a vessels deviation from a given normalroute
		/// </summary>
		/// <param name="vesselState">The current vesselstate, containing location of the state</param>
		/// <param name="routeID"> ID of the route the method fetches deviation from</param>
		/// <returns> A deviation object containing data about the deviation. </returns>
		public static async Task<Deviation> GetDeviationAsync(VesselState vesselState, int? routeID)
		{
			DeviationInput deviationInput = new DeviationInput(vesselState.Position.Y, vesselState.Position.X, vesselState.CourseOverGround, (double)vesselState.SpeedOverGround/10.0, routeID);
			string request = JsonConvert.SerializeObject(deviationInput);

			Uri url = new Uri(string.Format(GetBaseUrl() + "/api/get_deviation"));

			string response = await Post(url, request);
			if (response == null || response.Contains("result")) return null;
			return JsonConvert.DeserializeObject<Deviation>(response);
		}
		/// <summary>
		/// Fetches  warning info for a vessel 
		/// </summary>
		/// <param name="vesselState">The current vesselstate, containing location of the state</param>
		/// <returns>Info about the sector ahead of the vessel, 0 in both number of scerries and coastal points means there is no immediate danger. </returns>
		public static async Task<DangerOfRunningAgroundOutput> CheckForDangerOfRunningAground(VesselState vesselState)
		{
			//if (vesselState.SpeedOverGround < 1) return null;
			DangerOfRunningAgroundInput inputData = new DangerOfRunningAgroundInput(vesselState.Position.Y, vesselState.Position.X, vesselState.SpeedOverGround/10.0, vesselState.CourseOverGround);

			string request = JsonConvert.SerializeObject(inputData);

			Uri url = new Uri(string.Format(GetBaseUrl() + "/api/get_sector"));
			string response = null;
			try
			{
				response = await Post(url, request);
			}catch(Exception e)
			{
				Debug.WriteLine(e);
			}
		   

			if (response == null) return null;
			return JsonConvert.DeserializeObject<DangerOfRunningAgroundOutput>(response);
		}
	}
}
