﻿using GeoAPI.Geometries;
using Madart.OperationalUI.Common.Extensions.DataAccess;
using Madart.OperationalUI.DataAccess.Models;
using Madart.OperationalUI.DataAccess.Repositories;
using Madart.OperationalUI.MadAPI;
using Madart.OperationalUI.MadAPI.Models;
using Madart.OperationalUI.SignalR.Models;
using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Threading.Tasks;

namespace Madart.OperationalUI.WebApp.States
{
	public interface IVesselStateCollection
	{
		bool AddCurrentVesselState(CurrentVesselState currentVesselState);
		bool UpdateDeviation(int mmsiNo, Deviation newDeviation);
		List<Tuple<int?, double?>> GetNormalRouteIdRankList(int mmsiNo);
		bool UpdateNormalroute(int mmsiNo, List<Tuple<int?, double?>> normalRoutes);
		List<CurrentVesselState> GetAllCurrentVesselStates();
		CurrentVesselState GetCurrentVesselState(int mmsiNo);
		bool UpdateCurrentVesselState(CurrentVesselState newCurrentVesselState);
		bool UpdateDestination(int mmsiNo, string destination);
		bool UpdateTypeFiveShipType(int mmsiNo, string type5ShipType);
	}

	public class VesselStateCollection : IVesselStateCollection
	{


		/// <summary>
		/// A list containing the last update for each ship the application is handling, and corresponding normal route classified to the voyage.
		/// </summary>
		private static ConcurrentDictionary<int, CurrentVesselState> currentVesselStates = new ConcurrentDictionary<int, CurrentVesselState>(); 
		
		/// <summary>
		/// Adds a new CurrentVesselState to the dictionary
		/// </summary>
		/// <param name="currentVesselState"> The new CurrentVesselState</param>
		/// <returns>True if the adding went through, false if it failed, for example if the list already has a version of the CurrentVesselState</returns>
		public bool AddCurrentVesselState(CurrentVesselState currentVesselState)
		{
			return currentVesselStates.TryAdd(currentVesselState.MmsiNo, currentVesselState);
		}
	

		/// <summary>
		/// Updates a CurrentVesselState Deviation
		/// </summary>
		/// <param name="MmsiNo"> The MmsiNo for the relevant CurrentVesselState</param>
		/// <param name="newDeviation">The new Deviation that is to be added to the CurrentVesselState</param>
		/// <returns>True if update went through, false if there are no matches for the MmsiNo, or if the new Deviation is null</returns>
		public bool UpdateDeviation(int MmsiNo, Deviation newDeviation)
		{
			if (!currentVesselStates.ContainsKey(MmsiNo))
				return false;
			if (newDeviation == null) return false;
			currentVesselStates[MmsiNo].RelDistStd = newDeviation.rel_dist_std;
			currentVesselStates[MmsiNo].RelDistMax = newDeviation.rel_dist_max;
			currentVesselStates[MmsiNo].DistToP = newDeviation.disttoP;
			return true;
		}

		/// <summary>Used to get the normal route of a CurrentVesselState</summary>
		/// <param name="MmsiNo">The MmsiNo for the relevant CurrentVesselState</param>
		/// <returns>The normal route id for a vessel</returns>
		public List<Tuple<int?, double?>> GetNormalRouteIdRankList(int MmsiNo)
		{
			if (!currentVesselStates.ContainsKey(MmsiNo))
				return null;

			return currentVesselStates[MmsiNo].NormalRoutesIdRank;
		}





		/// <summary>Returns all  CurrentVesselStates that are cached on the server</summary>
		/// <returns>A list of all CurrentVesselStates</returns>
		public List<CurrentVesselState> GetAllCurrentVesselStates()
		{
			return currentVesselStates.Values.ToList();
		}



		/// <summary>Wipes all data in the dictionary</summary>
		public void ClearState()
		{
			currentVesselStates.Clear();
		}


		/// <summary>
		/// Updates the relevant CurrentVesselStates NormalRoute list
		/// </summary>
		/// <param name="MmsiNo">The MmsiNo for the relevant CurrentVesselState</param>
		/// <param name="normalRoutes">The new list of classified normal routes</param>
		/// <returns></returns>
		public bool UpdateNormalroute(int MmsiNo, List<Tuple<int?, double?>> normalRoutes)
		{
			if (!currentVesselStates.ContainsKey(MmsiNo) || normalRoutes == null || normalRoutes.Count ==0)
				return false;
			
			currentVesselStates[MmsiNo].NormalRoutesIdRank = normalRoutes;
			return true;
		}
		/// <summary>
		/// Gets a CurrentVesselState, given MmmsiNo
		/// </summary>
		/// <param name="MmsiNo">The MmsiNo for the relevant CurrentVesselState</param>
		/// <returns>The relevant CurrentVesselState</returns>
		public CurrentVesselState GetCurrentVesselState(int MmsiNo)
		{
			if (currentVesselStates.ContainsKey(MmsiNo))
				return currentVesselStates[MmsiNo];

			return null;
		}

		/// <summary>
		/// Updates info about a CurrentVesselState
		/// </summary>
		/// <param name="newCurrentVesselState">A CurrentVesselState containing updated data</param>
		/// <returns>True if update went through, false if the new CurrentVessel does not match any in the dictionary</returns>
		public bool UpdateCurrentVesselState(CurrentVesselState newCurrentVesselState)
		{
			if (!currentVesselStates.ContainsKey(newCurrentVesselState.MmsiNo)) return false;
			var temp = currentVesselStates[newCurrentVesselState.MmsiNo];
			newCurrentVesselState.VesselTypeGroupId = temp.VesselTypeGroupId;
			newCurrentVesselState.VesselTypeId = temp.VesselTypeId;
			newCurrentVesselState.NormalRoutesIdRank = temp.NormalRoutesIdRank;
			newCurrentVesselState.CurrentDestination = temp.CurrentDestination;
			newCurrentVesselState.Description = temp.Description;
			currentVesselStates[newCurrentVesselState.MmsiNo] = newCurrentVesselState;
			return true;
		}

		public bool UpdateDestination(int MmsiNo, string Destination)
		{
			if (!currentVesselStates.ContainsKey(MmsiNo) || string.IsNullOrWhiteSpace(Destination)) return false;
			currentVesselStates[MmsiNo].CurrentDestination = Destination;
			return true;
		}

		public bool UpdateTypeFiveShipType(int MmsiNo, string Type5ShipType)
		{
			if (!currentVesselStates.ContainsKey(MmsiNo) || string.IsNullOrWhiteSpace(Type5ShipType)) return false;
			currentVesselStates[MmsiNo].Description = Type5ShipType;
			return true;
		}
	}
}
